from django.views import View
from django.db.models import QuerySet
from django.core.exceptions import ImproperlyConfigured
from generic_table.generic_table import view

class GenericTableView(View):
    generic_table_view_kwargs = {}
    queryset = None
    model = None

    # https://github.com/django/django/blob/5a1cae3a5675c5733daf5949759476d65aa0e636/django/views/generic/list.py#L22C5-L48C1
    def get_queryset(self):
        if self.queryset is not None:
            queryset = self.queryset
            if isinstance(queryset, QuerySet):
                queryset = queryset.all()
        elif self.model is not None:
            queryset = self.model._default_manager.all()
        else:
            raise ImproperlyConfigured(
                "%(cls)s is missing a QuerySet. Define "
                "%(cls)s.model, %(cls)s.queryset, or override "
                "%(cls)s.get_queryset()." % {"cls": self.__class__.__name__}
            )
        return queryset


    def get(self, request, *args, **kwargs):
        return view(
            qs=self.get_queryset(),
            request=request,
            **self.generic_table_view_kwargs
        )



from django.conf import settings
from django.http import HttpRequest, HttpResponse


def api_key_middleware(get_response):
    def _(request: HttpRequest):
        request.current_user = request.headers.get("TS-USER", "system")

        if settings.DEBUG:
            return get_response(request)

        # Exclude health checks
        if request.path.endswith("health") or request.path.endswith("health/"):
            return get_response(request)

        if request.method == "OPTIONS":
            return HttpResponse("Good for preflight")

        user_filters = None
        try:
            user_filters = settings.USER_AUTH_FILTERS
        except AttributeError:
            pass

        if user_filters is not None:
            for f in user_filters:
                if f(request):
                    return get_response(request)

        user_api_key = None
        try:
            user_api_key = settings.USER_API_KEY
        except AttributeError:
            pass

        if user_api_key is not None:
            if request.headers.get("Authorization") == f"Bearer {settings.USER_API_KEY}":
                return get_response(request)

        res = HttpResponse("Not authenticated")
        res.status_code = 401
        return res

    return _
